clear
clc
%Monte Carlo simulations for the paper "Confidence Bands
%for Impulse Responses: Bonferroni versus Wald"
%by Helmut Luetkepohl, Anna Staszewska-Bystrova and Peter Winker, 2014.

randn('state',0);
rand('state',0);

mc=2000;     %number of Monte Carlo replications
n=2000;    %number of bootstrap replications
h=10;     %horizon (shock occurs at h=0)
sl=0.1;   %1-nominal coverage rate
tt=50;    %net sample size

lag_e=0;   %1 to estimate lag length in each MC replication, 0 to use true lag length
infoc=2;   %information criterion used to estimate number of lags: 1 for AICc, 2 for AIC, 3 for BIC
maxlag=10; % max. lag order

%**************************************************************************
alpha11=[-0.95 -0.9 -0.5 0 0.5 0.9 0.95 1]; %alternative values of alpha11 parameter

for x=1:length(alpha11)

    nlags1=1;
    nvar=2;

    sigma=zeros(2,2);
    sigma(1,1)=1;
    sigma(1,2)=0.3;
    sigma(2,1)=sigma(1,2);
    sigma(2,2)=1;

    b=zeros(3,2);

    b(2,1)=alpha11(x);
    b(2,2)=0.5;
    b(3,1)=0;
    b(3,2)=0.5;

    ct=tt+nlags1; %gross sample size

    %Response functions using true parameter values
    [irf_true]=ortirf(b,nlags1,nvar,h,sigma);

    traj=zeros(nvar^2,h+1);
    for i=1:h+1
        for j=1:nvar
            traj(nvar*(j-1)+1:nvar*j,i)=irf_true(nvar*i-nvar+1:nvar*i,j);
        end
    end

    %**************************************************************************
    W_cov=zeros(mc,nvar^2);             %coverage of the Wald (W) bands
    W_width=zeros(mc,nvar^2);           %width of the Wald (W) bands

    Wa_cov=zeros(mc,nvar^2);            %coverage of the Wa bands
    Wa_width=zeros(mc,nvar^2);          %width of the Wa bands

    WBa_cov=zeros(mc,nvar^2);           %coverage of the WBa bands
    WBa_width=zeros(mc,nvar^2);         %width of the WBa bands

    B_cov=zeros(mc,nvar^2);             %coverage of the Bonferroni (B) bands
    B_width=zeros(mc,nvar^2);           %width of the Bonferroni (B) bands

    Ba_cov=zeros(mc,nvar^2);            %coverage of the Ba bands
    Ba_width=zeros(mc,nvar^2);          %width of the Ba bands

    JWa_cov=zeros(mc,1);                %coverage of the JWa bands
    JWa_width=zeros(mc,1);              %width of the JWa bands

    JWBa_cov=zeros(mc,1);               %coverage of the JWBa bands
    JWBa_width=zeros(mc,1);             %width of the JWBa bands

    JB_cov=zeros(mc,1);                 %coverage of the JB bands
    JB_width=zeros(mc,1);               %width of the JB bands

    JBa_cov=zeros(mc,1);                %coverage of the JBa bands
    JBa_width=zeros(mc,1);              %width of the JBa bands

    JBstar_cov=zeros(mc,1);             %coverage of the JB* bands
    JBstar_width=zeros(mc,1);           %width of the JB* bands

    JBastar_cov=zeros(mc,1);            %coverage of the JBastar bands
    JBastar_width=zeros(mc,1);          %width of the JBastar band

    JW_cov=zeros(mc,1);                 %joint coverage of the W bands
    JW_width=zeros(mc,1);               %width of the W band

    for m=1:mc                          %MC replications
        [x m]

        %obtain a sample of data from the DGP
        [mx]=simdata(sigma,b,nlags1,nvar,ct);

        if lag_e==1
            nlags=lagest(mx,nvar,maxlag,infoc);
        else
            nlags=nlags1;
        end

        %bootstrap replicates of the response functions, responses based on
        %bias-corrected estimators and bootstrapped values of the Wald
        %statistic
        [bp1,resp,wb]=bpaths(mx,nlags,nvar,ct,h,n);

        [wsort,wind]=sort(wb);

        W=zeros(2*nvar^2,h+1);                          %W bands
        Wa=zeros(2*nvar^2,h+1);                         %Wa bands
        WBa=zeros(2*nvar^2,h+1);                        %WBa bands
        JWa=zeros(2*nvar^2,h+1);                        %JWa bands
        JWBa=zeros(2*nvar^2,h+1);                       %JWBa bands

        for c=1:nvar^2
            bb=bp1(n*(c-1)+1:c*n,:);
            bbs=bb(wind(1:round((1-sl)*n))',:);
            W(2*c-1,:)=min(bbs,[],1);
            W(2*c,:)=max(bbs,[],1);
            Wa(2*c-1:2*c,:)=adjW(nvar,sl,n,bb,wind,h);
            [WBa(2*c-1,:),WBa(2*c,:)]=adjB(h+1,W(2*c-1,:),W(2*c,:),bb,n,sl);
        end

        JWa=JadjW(nvar,sl,n,bp1,wind,h,W);
        JWBa=JadjB(h+1,W,bp1,n,sl,nvar);

        B=zeros(2*nvar^2,h+1);                          %B bands
        Ba=zeros(2*nvar^2,h+1);                         %Ba bands
        B1=zeros(2,h+1);

        for c=1:nvar^2
            bb1=bp1(n*(c-1)+1:c*n,:);
            if bb1(:,1)==zeros(n,1)
                B1(1,:)=quantile(bb1,sl/(2*(h)));
                B1(2,:)=quantile(bb1,1-sl/(2*(h)));
            else
                B1(1,:)=quantile(bb1,sl/(2*(h+1)));
                B1(2,:)=quantile(bb1,1-sl/(2*(h+1)));
            end
            B(2*c-1:2*c,:)=B1;
            [Ba(2*c-1,:),Ba(2*c,:)]=adjB(h+1,B(2*c-1,:),B(2*c,:),bb1,n,sl);
        end

        JB=zeros(2*nvar^2,h+1);                         %JB bands
        JBa=zeros(2*nvar^2,h+1);                        %JBa bands

        mm=nvar^2*h+nvar*((nvar+1)/2);
        for c=1:nvar^2
            bb1=bp1(n*(c-1)+1:c*n,:);
            JB(2*c-1,:)=quantile(bb1,sl/(2*(mm)));
            JB(2*c,:)=quantile(bb1,1-sl/(2*(mm)));
        end

        [JBa]=JadjB(h+1,JB,bp1,n,sl,nvar);

        JBstar=zeros(2*nvar^2,h+1);                     %JBstar bands     
        JBastar=zeros(2*nvar^2,h+1);                    %JBastar bands
        Bj1H=zeros(2,h+1);

        mm=nvar^2*h+nvar*((nvar+1)/2);
        if mm<=nvar^2*nlags+nvar*(nvar+1)/2
            JBstar=JB;
            JBastar=JBa;
        end
        if mm>nvar^2*nlags+nvar*(nvar+1)/2
            mmstar=nvar^2*nlags+nvar*(nvar+1)/2;
            for c=1:nvar^2
                bb1=bp1(n*(c-1)+1:c*n,:);
                JBstar(2*c-1:2*c,:)=JB_star(bb1,sl,mmstar,h,n,nvar,nlags);
            end
        end

        [JBastar]=JadjB(h+1,JBstar,bp1,n,sl,nvar);

        %check coverage wrt individual IRFs and calculate width
        for z=1:nvar^2
            W_cov(m,z)=inside(1,traj(z,:),W(2*z-1,:),W(2*z,:));
            Wa_cov(m,z)=inside(1,traj(z,:),Wa(2*z-1,:),Wa(2*z,:));
            WBa_cov(m,z)=inside(1,traj(z,:),WBa(2*z-1,:),WBa(2*z,:));
            B_cov(m,z)=inside(1,traj(z,:),B(2*z-1,:),B(2*z,:));
            Ba_cov(m,z)=inside(1,traj(z,:),Ba(2*z-1,:),Ba(2*z,:));

            W_width(m,z)=sos(W(2*z-1,:),W(2*z,:));
            Wa_width(m,z)=sos(Wa(2*z-1,:),Wa(2*z,:));
            WBa_width(m,z)=sos(WBa(2*z-1,:),WBa(2*z,:));
            B_width(m,z)=sos(B(2*z-1,:),B(2*z,:));
            Ba_width(m,z)=sos(Ba(2*z-1,:),Ba(2*z,:));
        end

        %check joint coverage wrt all IRFs and calculate width
        JW_cov(m,1)=inside_joint(traj,W);
        JW_width(m,1)=sos_joint(W);

        JWa_cov(m,1)=inside_joint(traj,JWa);
        JWa_width(m,1)=sos_joint(JWa);

        JWBa_cov(m,1)=inside_joint(traj,JWBa);
        JWBa_width(m,1)=sos_joint(JWBa);

        JB_cov(m,1)=inside_joint(traj,JB);
        JB_width(m,1)=sos_joint(JB);

        JBa_cov(m,1)=inside_joint(traj,JBa);
        JBa_width(m,1)=sos_joint(JBa);

        JBstar_cov(m,1)=inside_joint(traj,JBstar);
        JBstar_width(m,1)=sos_joint(JBstar);

        JBastar_cov(m,1)=inside_joint(traj,JBastar);
        JBastar_width(m,1)=sos_joint(JBastar);
    end

    %prepare "Summary" file with results, saved in the "Results" folders
    out=output(mc,n,tt,sl,h,lag_e,x,alpha11,W_cov,Wa_cov,WBa_cov,B_cov,Ba_cov,...
        W_width,Wa_width,WBa_width,B_width,Ba_width,...
        JW_cov,JWa_cov,JWBa_cov,JB_cov,JBa_cov,JBstar_cov,JBastar_cov,...
        JW_width,JWa_width,JWBa_width,JB_width,JBa_width,JBstar_width,JBastar_width);

end
